# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_logbookForm.ui'
#
# Created: Wed Jun 25 14:54:33 2014
#      by: PyQt4 UI code generator 4.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_logbookForm(object):
    def setupUi(self, logbookForm):
        logbookForm.setObjectName(_fromUtf8("logbookForm"))
        logbookForm.resize(549, 189)
        self.gridLayout = QtGui.QGridLayout(logbookForm)
        self.gridLayout.setContentsMargins(4, 4, 4, 0)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.headerLayout = QtGui.QHBoxLayout()
        self.headerLayout.setObjectName(_fromUtf8("headerLayout"))
        self.userLayout = QtGui.QHBoxLayout()
        self.userLayout.setSpacing(0)
        self.userLayout.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.userLayout.setContentsMargins(0, -1, 0, -1)
        self.userLayout.setObjectName(_fromUtf8("userLayout"))
        self.userLabel = QtGui.QLabel(logbookForm)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.userLabel.sizePolicy().hasHeightForWidth())
        self.userLabel.setSizePolicy(sizePolicy)
        self.userLabel.setMinimumSize(QtCore.QSize(40, 0))
        self.userLabel.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.userLabel.setObjectName(_fromUtf8("userLabel"))
        self.userLayout.addWidget(self.userLabel)
        self.userName = QtGui.QLineEdit(logbookForm)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(6)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.userName.sizePolicy().hasHeightForWidth())
        self.userName.setSizePolicy(sizePolicy)
        self.userName.setMinimumSize(QtCore.QSize(120, 0))
        self.userName.setMaximumSize(QtCore.QSize(500, 16777215))
        self.userName.setSizeIncrement(QtCore.QSize(2, 0))
        self.userName.setObjectName(_fromUtf8("userName"))
        self.userLayout.addWidget(self.userName)
        self.userLayout.setStretch(0, 0)
        self.userLayout.setStretch(1, 6)
        self.headerLayout.addLayout(self.userLayout)
        spacerItem = QtGui.QSpacerItem(0, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.headerLayout.addItem(spacerItem)
        self.multiLogLayout = QtGui.QVBoxLayout()
        self.multiLogLayout.setObjectName(_fromUtf8("multiLogLayout"))
        self.headerLayout.addLayout(self.multiLogLayout)
        self.headerLayout.setStretch(0, 6)
        self.headerLayout.setStretch(1, 6)
        self.headerLayout.setStretch(2, 6)
        self.gridLayout.addLayout(self.headerLayout, 0, 0, 1, 1)
        self.titleLayout = QtGui.QHBoxLayout()
        self.titleLayout.setSpacing(0)
        self.titleLayout.setObjectName(_fromUtf8("titleLayout"))
        self.titleLabel = QtGui.QLabel(logbookForm)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.titleLabel.sizePolicy().hasHeightForWidth())
        self.titleLabel.setSizePolicy(sizePolicy)
        self.titleLabel.setMinimumSize(QtCore.QSize(40, 0))
        self.titleLabel.setObjectName(_fromUtf8("titleLabel"))
        self.titleLayout.addWidget(self.titleLabel)
        self.titleEntry = QtGui.QLineEdit(logbookForm)
        self.titleEntry.setObjectName(_fromUtf8("titleEntry"))
        self.titleLayout.addWidget(self.titleEntry)
        self.gridLayout.addLayout(self.titleLayout, 1, 0, 1, 1)
        self.textLayout = QtGui.QHBoxLayout()
        self.textLayout.setSpacing(0)
        self.textLayout.setObjectName(_fromUtf8("textLayout"))
        self.textLabel = QtGui.QLabel(logbookForm)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textLabel.sizePolicy().hasHeightForWidth())
        self.textLabel.setSizePolicy(sizePolicy)
        self.textLabel.setMinimumSize(QtCore.QSize(40, 0))
        self.textLabel.setObjectName(_fromUtf8("textLabel"))
        self.textLayout.addWidget(self.textLabel)
        self.textEntry = QtGui.QTextEdit(logbookForm)
        self.textEntry.setObjectName(_fromUtf8("textEntry"))
        self.textLayout.addWidget(self.textEntry)
        self.gridLayout.addLayout(self.textLayout, 2, 0, 1, 1)
        self.footerLayout = QtGui.QHBoxLayout()
        self.footerLayout.setSpacing(10)
        self.footerLayout.setContentsMargins(-1, -1, -1, 10)
        self.footerLayout.setObjectName(_fromUtf8("footerLayout"))
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.footerLayout.addItem(spacerItem1)
        self.imageFrame = QtGui.QLabel(logbookForm)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imageFrame.sizePolicy().hasHeightForWidth())
        self.imageFrame.setSizePolicy(sizePolicy)
        self.imageFrame.setMinimumSize(QtCore.QSize(50, 50))
        self.imageFrame.setMaximumSize(QtCore.QSize(50, 50))
        self.imageFrame.setFrameShadow(QtGui.QFrame.Plain)
        self.imageFrame.setLineWidth(0)
        self.imageFrame.setText(_fromUtf8(""))
        self.imageFrame.setObjectName(_fromUtf8("imageFrame"))
        self.footerLayout.addWidget(self.imageFrame)
        spacerItem2 = QtGui.QSpacerItem(93, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.footerLayout.addItem(spacerItem2)
        self.resetButton = QtGui.QPushButton(logbookForm)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.resetButton.sizePolicy().hasHeightForWidth())
        self.resetButton.setSizePolicy(sizePolicy)
        self.resetButton.setMinimumSize(QtCore.QSize(80, 0))
        self.resetButton.setMaximumSize(QtCore.QSize(80, 16777215))
        self.resetButton.setObjectName(_fromUtf8("resetButton"))
        self.footerLayout.addWidget(self.resetButton)
        spacerItem3 = QtGui.QSpacerItem(30, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.footerLayout.addItem(spacerItem3)
        self.cancelButton = QtGui.QPushButton(logbookForm)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cancelButton.sizePolicy().hasHeightForWidth())
        self.cancelButton.setSizePolicy(sizePolicy)
        self.cancelButton.setMinimumSize(QtCore.QSize(80, 0))
        self.cancelButton.setMaximumSize(QtCore.QSize(80, 16777215))
        self.cancelButton.setObjectName(_fromUtf8("cancelButton"))
        self.footerLayout.addWidget(self.cancelButton)
        self.submitButton = QtGui.QPushButton(logbookForm)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.submitButton.sizePolicy().hasHeightForWidth())
        self.submitButton.setSizePolicy(sizePolicy)
        self.submitButton.setMinimumSize(QtCore.QSize(80, 0))
        self.submitButton.setMaximumSize(QtCore.QSize(80, 16777215))
        self.submitButton.setDefault(True)
        self.submitButton.setObjectName(_fromUtf8("submitButton"))
        self.footerLayout.addWidget(self.submitButton)
        self.gridLayout.addLayout(self.footerLayout, 3, 0, 1, 1)

        self.retranslateUi(logbookForm)
        QtCore.QMetaObject.connectSlotsByName(logbookForm)

    def retranslateUi(self, logbookForm):
        logbookForm.setWindowTitle(_translate("logbookForm", "Logbook Entry", None))
        self.userLabel.setText(_translate("logbookForm", "User:", None))
        self.titleLabel.setText(_translate("logbookForm", "Title:", None))
        self.textLabel.setText(_translate("logbookForm", "Text:", None))
        self.resetButton.setText(_translate("logbookForm", "Reset", None))
        self.cancelButton.setText(_translate("logbookForm", "Cancel", None))
        self.submitButton.setText(_translate("logbookForm", "Submit", None))

